@@ -1,11 +1,8 @@
 #![deny(clippy::all, clippy::pedantic)]
 #![allow(
   clippy::doc_markdown,
-  clippy::empty_enum,
   clippy::enum_glob_use,
-  clippy::if_not_else,
   clippy::missing_errors_doc,
-  clippy::needless_lifetimes,
   clippy::needless_pass_by_value,
   clippy::non_ascii_literal,
   clippy::shadow_unrelated,
